% a predicate waits for both its arguments to become ground
% the first argument is already instantiated with a long list
% the second becomes succesively instantiated with a long list.
% if the checking code always checks the first argument, 
% then the next etc., it will have a lot to do. However, if it checks
% in the other direction or remembers that the first argument is ground,
% it will a lot faster (three times or more).

top:-
	read(N,[]),
	cputime(T0),
	gen(N,L1),
	q(L1,L2),
	p(L1,L2),
	cputime(T1),
	Time is T1-T0,
	write((time=Time,p(L1,L2))),nl.

p([],[]).
p([A|X],[B|Y]):- change(A,B), p(X,Y).

change(A,B):- B is A-1.

gen(0,[]).
gen(N,[N|T]):-N1 is N-1, gen(N1,T).

delay q(ground,ground).
q(_,_).
